linux-block.vger.kernel.org archive mirror
 help / color / mirror / Atom feed
From: Damien Le Moal <damien.lemoal@wdc.com>
To: linux-block@vger.kernel.org, Jens Axboe <axboe@kernel.dk>
Subject: [PATCH 2/7] block: Change elevator_init_mq() to always succeed
Date: Fri, 23 Aug 2019 09:15:23 +0900	[thread overview]
Message-ID: <20190823001528.5673-3-damien.lemoal@wdc.com> (raw)
In-Reply-To: <20190823001528.5673-1-damien.lemoal@wdc.com>

If the default elevator chosen is mq-deadline, elevator_init_mq() may
return an error if the mq-deadline initialization fails, leading to
blk_mq_init_allocated_queue() returning an error, which in turn will
cause the block device initialization to fail.

Instead of taking such extreme measure, handle mq-deadline
initialization failures in the same manner as if mq-deadline being not
available (no module to load), that is, default to the "none" scheduler.
With this change, elevator_init_mq() return type can be changed to void.

Signed-off-by: Damien Le Moal <damien.lemoal@wdc.com>
---
 block/blk-mq.c   |  8 +-------
 block/blk.h      |  2 +-
 block/elevator.c | 17 ++++++++++-------
 3 files changed, 12 insertions(+), 15 deletions(-)

diff --git a/block/blk-mq.c b/block/blk-mq.c
index 556c774a0f0d..274e168c8535 100644
--- a/block/blk-mq.c
+++ b/block/blk-mq.c
@@ -2846,8 +2846,6 @@ static unsigned int nr_hw_queues(struct blk_mq_tag_set *set)
 struct request_queue *blk_mq_init_allocated_queue(struct blk_mq_tag_set *set,
 						  struct request_queue *q)
 {
-	int ret = -ENOMEM;
-
 	/* mark the queue as mq asap */
 	q->mq_ops = set->ops;
 
@@ -2908,14 +2906,10 @@ struct request_queue *blk_mq_init_allocated_queue(struct blk_mq_tag_set *set,
 	blk_mq_add_queue_tag_set(set, q);
 	blk_mq_map_swqueue(q);
 
-	ret = elevator_init_mq(q);
-	if (ret)
-		goto err_tag_set;
+	elevator_init_mq(q);
 
 	return q;
 
-err_tag_set:
-	blk_mq_del_queue_tag_set(q);
 err_hctxs:
 	kfree(q->queue_hw_ctx);
 	q->nr_hw_queues = 0;
diff --git a/block/blk.h b/block/blk.h
index de6b2e146d6e..ddb292bb6caf 100644
--- a/block/blk.h
+++ b/block/blk.h
@@ -184,7 +184,7 @@ void blk_account_io_done(struct request *req, u64 now);
 
 void blk_insert_flush(struct request *rq);
 
-int elevator_init_mq(struct request_queue *q);
+void elevator_init_mq(struct request_queue *q);
 int elevator_switch_mq(struct request_queue *q,
 			      struct elevator_type *new_e);
 void __elevator_exit(struct request_queue *, struct elevator_queue *);
diff --git a/block/elevator.c b/block/elevator.c
index 1ed2710f1950..7fff06751633 100644
--- a/block/elevator.c
+++ b/block/elevator.c
@@ -603,19 +603,19 @@ static inline bool elv_support_iosched(struct request_queue *q)
 
 /*
  * For blk-mq devices supporting IO scheduling, we default to using mq-deadline,
  * if available, for single queue devices. If deadline isn't available OR we
  * have multiple queues, default to "none".
  */
-int elevator_init_mq(struct request_queue *q)
+void elevator_init_mq(struct request_queue *q)
 {
 	struct elevator_type *e;
 	int err = 0;
 
 	if (!elv_support_iosched(q))
-		return 0;
+		return;
 
 	if (q->nr_hw_queues != 1)
-		return 0;
+		return;
 
 	/*
 	 * q->sysfs_lock must be held to provide mutual exclusion between
@@ -630,11 +630,14 @@ int elevator_init_mq(struct request_queue *q)
 		goto out_unlock;
 
 	err = blk_mq_init_sched(q, e);
-	if (err)
+	if (err) {
+		pr_warn("\"%s\" elevator initialization failed, "
+			"falling back to \"none\"\n", e->elevator_name);
 		elevator_put(e);
+	}
+
 out_unlock:
 	mutex_unlock(&q->sysfs_lock);
-	return err;
 }
 
 
@@ -603,19 +603,19 @@ static inline bool elv_support_iosched(struct request_queue *q)
 /*
  * For blk-mq devices supporting IO scheduling, we default to using mq-deadline,
- * if available, for single queue devices. If deadline isn't available OR we
- * have multiple queues, default to "none".
+ * if available, for single queue devices. If deadline isn't available OR
+ * deadline initialization fails OR we have multiple queues, default to "none".
  */
-- 
2.21.0


  parent reply	other threads:[~2019-08-23  0:15 UTC|newest]

Thread overview: 14+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2019-08-23  0:15 [PATCH 0/7] Elevator cleanups and improvements Damien Le Moal
2019-08-23  0:15 ` [PATCH 1/7] block: Cleanup elevator_init_mq() use Damien Le Moal
2019-08-23  8:08   ` Johannes Thumshirn
2019-08-23  0:15 ` Damien Le Moal [this message]
2019-08-23  8:54   ` [PATCH 2/7] block: Change elevator_init_mq() to always succeed Johannes Thumshirn
2019-08-23  0:15 ` [PATCH 3/7] block: Remove sysfs lock from elevator_init_rq() Damien Le Moal
2019-08-23  9:00   ` Johannes Thumshirn
2019-08-23 20:32   ` Bart Van Assche
2019-08-23  0:15 ` [PATCH 4/7] block: Introduce elevator features Damien Le Moal
2019-08-23 11:19   ` Johannes Thumshirn
2019-08-23  0:15 ` [PATCH 5/7] block: Introduce zoned block device elevator feature Damien Le Moal
2019-08-23 11:41   ` Johannes Thumshirn
2019-08-23  0:15 ` [PATCH 6/7] block: Improve default elevator selection Damien Le Moal
2019-08-23  0:15 ` [PATCH 7/7] block: Delay default elevator initialization Damien Le Moal

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20190823001528.5673-3-damien.lemoal@wdc.com \
    --to=damien.lemoal@wdc.com \
    --cc=axboe@kernel.dk \
    --cc=linux-block@vger.kernel.org \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).